//
// FPGA PACMAN video double scan driver
//
// Version : beta1
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk. 
//
//

module dbl_scan(pclk,dclk,rgb8_in,hs_in,vs_in,rgb8_out,hs_out,vs_out);
input pclk;             // 6MHz  input pixel clock
input dclk;             // 12Mhz output pixel clock
input [7:0] rgb8_in;    // RGB   input
input hs_in;            // HSYNC input (15KHz)
input vs_in;            // VSYNC input (60Hz)
output [7:0] rgb8_out;  // RGB output
output hs_out;          // HSYNC output (30KHz)
output vs_out;          // VSYNC output (60Hz)

//
// input timming
//
reg [8:0] hs_endpos; // end of hsync position
reg [8:0] hpos_i;    // input capture postion
reg ihs;
wire ic_flip = ihs^hs_in; // flip hsync
always @(posedge pclk)
begin
	hpos_i <= ic_flip&hs_in ? 0 : hpos_i + 1;

	if(ic_flip&~hs_in)
		hs_endpos = hpos_i;
end

//
//output timming
//
reg [8:0] hpos_o;
reg ohs;
wire oc_r = (ohs^hs_in)&hs_in; // raise hsync 
always @(posedge dclk)
begin
	if( oc_r || (hpos_o == 384-1) )
		hpos_o <= 0;
	 else
		hpos_o <= hpos_o + 1;
	ohs <= hs_in;
end

//
// RGB capture(portA) & output(portB)
//
RAMB4_S8_S8 double_scan_ram (
	.CLKA(pclk),.ADDRA(hpos_i),.DOA(),.DIA(rgb8_in)     ,.WEA(1'b1),.ENA(1'b1),.RSTA(1'b0),
	.CLKB(dclk),.ADDRB(hpos_o),.DOB(rgb8_out),.DIB(4'h0),.WEB(1'b0),.ENB(1'b1),.RSTB(1'b0));

//
// vsync cut off
//
// 1 HSYNC_IN delay & HSYNC pulse width = 4xHSYNC(in)
//
reg [2:0] vs_cnt;
wire vs_mask = (vs_cnt != 3'b000);
always @(posedge hs_in)
begin
	if(~vs_in)
	begin
		vs_cnt <= 0;
	end else begin
		if(vs_cnt <= 4)
 	   	vs_cnt <= vs_cnt + 1;
	end
end


// converted SYNC signal
assign vs_out = (vs_cnt !=0) & (vs_cnt != 4+1);
assign hs_out = (hpos_o < 8);

endmodule
