//
// FPGA PACMAN waveform sound
//
// Version : beta1
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk. 
//
//

module sound(A,DI,WR0,WR1,PCLK,SOUND_ON,HC,DAO);
input [3:0] A;
input [3:0] DI;
input WR0;
input WR1;
input PCLK; // 2 x HC[0]
input SOUND_ON;
input [5:0] HC;
output [9:0] DAO;

//
// sampling rate : 3072000/32 =96KHz
//

//
// sound registers
//
reg [19:0]  acc0,frq0; //5bytes
reg [15:0]  acc1,frq1,acc2,frq2; //4bytes
reg [3:0]  wav0,vol0,wav1,vol1,wav2,vol2;

//
// sound generator
//
reg [8:0] SA;
wire [3:0] SD;
reg [3:0] vol;
reg [6:0] wav;
reg [9:0] pcm_acc,pcm_out;
always @(posedge PCLK)
begin
	if(HC[0] == 1'b0)
	begin // CPU DATA latch phase
		if(WR0)
		begin
			case(A)
			4'h0: acc0[ 3: 0] <= DI;
			4'h1: acc0[ 7: 4] <= DI;
			4'h2: acc0[11: 8] <= DI;
			4'h3: acc0[15:12] <= DI;
			4'h4: acc0[19:16] <= DI;
			4'h5: wav0        <= DI;
			4'h6: acc1[ 3: 0] <= DI;
			4'h7: acc1[ 7: 4] <= DI;
			4'h8: acc1[11: 8] <= DI;
			4'h9: acc1[15:12] <= DI;
			4'ha: wav1        <= DI;
			4'hb: acc2[ 3: 0] <= DI;
			4'hc: acc2[ 7: 4] <= DI;
			4'hd: acc2[11: 8] <= DI;
			4'he: acc2[15:12] <= DI;
			4'hf: wav2        <= DI;
			endcase
		end
		// addr 5050-505f
		if(WR1)
		begin
			case(A)
			4'h0: frq0[ 3: 0] <= DI;
			4'h1: frq0[ 7: 4] <= DI;
			4'h2: frq0[11: 8] <= DI;
			4'h3: frq0[15:12] <= DI;
			4'h4: frq0[19:16] <= DI;
			4'h5: vol0        <= DI;
			4'h6: frq1[ 3: 0] <= DI;
			4'h7: frq1[ 7: 4] <= DI;
			4'h8: frq1[11: 8] <= DI;
			4'h9: frq1[15:12] <= DI;
			4'ha: vol1        <= DI;
			4'hb: frq2[ 3: 0] <= DI;
			4'hc: frq2[ 7: 4] <= DI;
			4'hd: frq2[11: 8] <= DI;
			4'he: frq2[15:12] <= DI;
			4'hf: vol2        <= DI;
			endcase
		end
	end else begin

	// channel source
	case(HC[5:1])
	// CH1
	4'h4: begin // 0-4
		wav <= wav0;
		acc0 <= acc0 + frq0;
		SA   <= { wav0 , acc0[19:15] };
		// update mixed PCM value
		pcm_out <= pcm_acc;
		pcm_acc <= 0;
		end
	4'h5:begin wav <= { 3'b000,SD }; vol <= vol0; end
	// CH2
	4'h9:begin // 6-9
		acc1 <= acc1 + frq1;
		SA   <= { wav1 , acc1[15:11] };
		end
	4'ha:begin wav <= { 3'b000,SD }; vol <= vol1; end
	// CH3
	4'he:begin // b-e
		acc2 <= acc2 + frq2;
		SA   <= { wav2 , acc2[15:11] };
		end
	4'hf:begin wav <= { 3'b000,SD }; vol <= vol2; end
	endcase

	end
	
	// PCM accumurator
	if(vol != 4'b0000)
	begin
		if(vol[0])
			pcm_acc <= pcm_acc + wav;
		vol <= { 1'b0 , vol[3:1] };
		wav <= { wav[5:0] , 1'b0};
	end
end

//
// sound ROM
//
wire [9:0] roma = {1'b0,SA};
wire [7:0] romd;
RAMB4_S8 WAVE_ROM00 (.CLK(PCLK),.ADDR(roma),.DO(romd),.DI(4'h0),.EN(1'b1),.WE(1'b0),.RST(1'b0));
assign SD = romd[3:0];
//assign SD = SA[4] ? 8'hff : 8'h00;
//
// output
//
// HC[5] = 192KHz

//assign DAO = SOUND_ON ? 8'h0 : pcm_out;
assign DAO = pcm_out;

endmodule
