//
// FPGA PACMAN TOP
//
// Version : beta1
//
// Copyright(c) 2002 Tatsuyuki Satoh , All rights reserved
//
// Important !
//
// This program is freeware for non-commercial use. 
// An author does no guarantee about this program.
// You can use this under your own risk. 
//
//

module top(
// check
//	Z80_RD,Z80_CLK,
// VGA out
	VGA_R,VGA_G,VGA_B,
	n_HSYNC,n_VSYNC,
//CPU interface
//ROM interface
	RA,RD,RCS,ROE,RWE,
//Sound
	DAC_L,DAC_R,
//Contyroll switches
	PSW,
// 18.432MHz general CLOCK input
	PCLK,
);

//I/O assign
output DAC_L,DAC_R;

input [3:0] PSW;
input PCLK; // 18.432MHz

// VGA out
output [4:0] VGA_R , VGA_G , VGA_B;
output n_HSYNC,n_VSYNC;

// code / pattern ROM interface
output [18:0] RA;
inout [7:0] RD;
output RCS,ROE,RWE;

//CPU interface
//input [15:0] XA;
//input [7:0] XD;
//inout XMREQ,XIORQ,XM1,XRD,XWR;
//output XCLK;

//
// clock generator
//
wire CLK18_432M = PCLK;
wire CLK12_288M;
wire CLK6_144M;

clock BASE_CLOCK(
	.CLKIN(CLK18_432M),
	.CLK_2_3(CLK12_288M),
	.CLK_1_6(CLK6_144M),
	.CLK_1_6_n()
);
// base & video pixel clock
wire pixel_clk;
BUFG PIXEL_CLOCK(.I(CLK6_144M),.O(pixel_clk));

// ticker , clock  counter (PCLK = 18.432MHz)
// ticker[16] = @140Hz
// ticker[24] = @1Hz
reg  [39:0] ticker;
wire CLK1Hz;

assign CLK1Hz =ticker[23]; // @1Hz
//always @(posedge PCLK)
always @(posedge CLK18_432M)
begin
		ticker <= ticker + 1;   // 18.432MHz
end

//
// Z80 core
//
wire [7:0] ZDO,ZDI;
//output
wire [15:0] ZA;
wire ZMREQ,ZIORQ,ZM1,ZRD,ZWR;
wire ZWO;
//input
wire Z80CLK2,Z80CLK,ZINT,ZNMI,ZRESET,ZWAIT;

Z80EX u_z80(
	.CLK2X(Z80CLK2),
	.CLK(Z80CLK),
	.RESET_N(ZRESET),
	.INT_N(ZINT),
	.NMI_N(ZNMI),
	.A(ZA),
	.DOUT(ZDO),
	.DIN(ZDI),
	.M1_N(ZM1),
	.MREQ_N(ZMREQ),
	.IORQ_N(ZIORQ),
	.RD_N(ZRD),
	.WR_N(ZWR),
	.WAIT_N(ZWAIT),
	.BUSWO(ZWO),
	.RFSH_N(),
	.HALT_N()
);
assign ZRESET = PSW[0]|PSW[1]|PSW[2]|PSW[3];
assign ZNMI = 1'b1;

//
//
// address decoder
//
//
wire flip;
wire in0,in1,dsw1,dsw2;
wire wr0,wr1;
wire snd_on;
wire csxyram , csvram;
wire csrom;
adec pacman_adec(
// inputs
	.A(ZA),
	.n_MREQ(ZMREQ),
	.D0I(ZDO[0]),
	.n_RD(ZRD),
	.n_WR(ZWR),
	.RST(1'b0),
// decode output
	.ROM(csrom),
	.RAM(csvram),
	.IN0(in0),
	.IN1(in1),
	.DSW1(dsw1),
	.DSW2(dsw2),
	.WR0(wr0),
	.WR1(wr1),	// 5050-505f.W sound frequency   / volume
	.WR2(csxyram),	// 5060-506f.W object X,Y RAM
	.WDT(),	// 50c0-50ff.W Watchdog timer reset
// latched register
	.IRQ(irqen),	// 5000.W      IRQ enable (clear)
	.SOUND_ON(snd_on),// 5001.W      sound enable
	.FLIP(flip),	// 5003.W      flip
	.LAMP1P(),	// 5004.W      1P start lamp
	.LAMP2P(),	// 5005.W      2P start lamp
	.COINLK(),	// 5006.W      coin lockout
	.COINUP()	// 5007.W      coin countup
);

//
// CPU PROM to shared ROM interface
//
wire [18:0] cpu_a;
reg [7:0] cpu_rd;
wire [7:0] rom_do = csrom ? cpu_rd : 8'h00;
assign cpu_a[15:0]  = ZA;
assign cpu_a[18:16] = 0;

//
//
// controller IN / DIP switch I/F
//
//
wire L1 = PSW[2];
wire R1 = PSW[3];
wire U1 = PSW[0];
wire D1 = PSW[1];
//
wire L2 = L1;
wire R2 = R1;
wire U2 = U1;
wire D2 = D1;
wire S1 = L1|~R1;
wire S2 = R1|~L1;
wire C1 = L1|R1|U1|~D1;
wire C2 = L1|R1|~U1|D1;
wire [7:0] inp_do;
inport pacman_inport(
// inputs
	.LEFT1(L1),
	.RIGHT1(R1),
	.UP1(U1),
	.DOWN1(D1),
	.LEFT2(L2),
	.RIGHT2(R2),
	.UP2(U2),
	.DOWN2(D2),
	.START1(S1),
	.START2(S2),
	.COIN1(C1),
	.COIN2(C2),
	.CREDIT(1'b1),
	.TEST(1'b1),
	.TABLE(1'b1),
//enables
	.IN0(in0),
	.IN1(in1),
	.DSW1(dsw1),
	.DSW2(dsw2),
//output
	.DO(inp_do)	
);

//
//
// VIDEO hardware interface
//
//
wire [8:0] hcnt;
wire hsync,vsync;
wire [7:0] vdo;
wire vblank;
wire [2:0] red;
wire [2:0] green;
wire [1:0] blue;
reg [7:0] pd;   //pattern ROM data
wire [12:0] pa; //patterm ROM address

video vh(
	.A(ZA[11:0]),
	.csvram(csvram),
	.csxyram(csxyram),
	.n_wr(ZWR),
	.n_rd(ZRD),
	.di(ZDO),
	.do(vdo),
	.pclk(pixel_clk),
	.flip(flip),
	.vb(vblank),
	.hc(hcnt),
	.hs(hsync),
	.vs(vsync),
	.r(red),
	.g(green),
	.b(blue),
	.CA(pa),
	.CD(RD)
);

// CPU clock assign
//BUFG ZCLK(.I(~hcnt[0]),.O(Z80CLK2)); // 3MHz

BUFG ZCLK(.I(~hcnt[0]),.O(Z80CLK2)); // 3MHz

reg zclk;
always @(posedge Z80CLK2)
begin
	zclk <= ~zclk;
end
BUFG ZCLK1(.I(zclk),.O(Z80CLK)); // 3MHz
//assign Z80CLK  = zclk;

//assign Z80CLK  = ~hcnt[1];
assign ZWAIT   = 1'b1;

//
//
// IRQ & IRQ vector
//
//
wire [7:0] irq_do;
wire vector_enable;
irq pacman_irqvector(
//inputs
	.n_M1(ZM1),
	.n_IORQ(ZIORQ),
	.IRQEN(irqen),
	.n_IRQTRG(vblank),
	.DI(ZDO),
//outputs
	.n_IRQ(ZINT),
	.DO(irq_do),
	.DE(vector_enable)
);

//
//
// sound
//
//
wire [9:0] wave_out;
sound sound(
	.A(ZA[3:0]),
	.DI(ZDO[3:0]),
	.WR0(wr0),
	.WR1(wr1),
	.PCLK(pixel_clk),
	.SOUND_ON(snd_on),
	.HC(hcnt[5:0]),
//
	.DAO(wave_out)
);

//
// CPU code / patterm ROM interface
//
// interleave CPU address & video pattern address
//
reg [18:0] rom_a;
always @(posedge pixel_clk)
begin
	// latch CPU data & switch video address
	if( hcnt[0] == 2'b0)
	begin
		cpu_rd <= RD;
		rom_a  <= { 6'b000011 ,  pa}; //06000-07fff
	end

	// latch pixel data & switch cpu address
	if( hcnt[0] == 2'b1)
	begin
		pd    <= RD;
		rom_a <= cpu_a;
	end
end
assign RA  = rom_a;
assign RD  = 8'bzzzzzzzz;
assign RCS = 1'b0;
assign RWE = 1'b1;
assign ROE = 1'b0;

//
// data bus miltiplexer
//
assign ZDI = csvram ? vdo : (rom_do | inp_do | irq_do);
//assign ZDI = vdo | rom_do | inp_do | irq_do;

//
// VGA output interface
//
// R,G,B = open gate   , series register drive
//
//
// HS,HV = direct connect with 47ohm DUMP/safe Register
//


//
// DOUBLE SCAN CONVERSION
//
//dbl_scan double_scan();
wire hs_up , vs_up;
wire [7:0] rgb_in = { red,green,blue };
wire [7:0] rgb_up;
wire [2:0] red_up   = rgb_up[7:5];
wire [2:0] green_up = rgb_up[4:2];
wire [1:0] blue_up  = rgb_up[1:0];

dbl_scan double_scan(
	.pclk(pixel_clk),
	.dclk(CLK12_288M),
	.rgb8_in(rgb_in),
	.hs_in(hsync),
	.vs_in(vsync),
	.rgb8_out(rgb_up),
	.hs_out(hs_up),
	.vs_out(vs_up)
);
 

// VGA REG output , open gate
assign VGA_R[4] = red_up[2]; // 500ohm ( 2K/4 )
assign VGA_R[3] = red_up[1]; // 1Kohm  ( 2K/2 )
assign VGA_R[2] = red_up[0]; // 2Kohm
assign VGA_R[1] = 1'bz;   // 4Kohm  ( 2Kx2 )
assign VGA_R[0] = 1'bz;   // 8Kohm  ( 2Kx4 )
// VGA GREEN output , open gate
assign VGA_G[4] = green_up[2];
assign VGA_G[3] = green_up[1];
assign VGA_G[2] = green_up[0];
assign VGA_G[1] = 1'bz;
assign VGA_G[0] = 1'bz;
// VGA BLUE output , open gate
assign VGA_B[4] = blue_up[1];
assign VGA_B[3] = blue_up[0];
assign VGA_B[2] = 1'bz;
assign VGA_B[1] = 1'bz;
assign VGA_B[0] = 1'bz;

//
// VGA SYNC signal interface
//
assign n_HSYNC = hs_up ? 1'b0 : 1'b1;
assign n_VSYNC = vs_up ? 1'b0 : 1'b1;

//
// SOUND interface
//
// Sigma Delta :  sampling rate = 37.53KHz
wire dac_out;
dac dac(.DACout(dac_out),.DACin(wave_out[9:2]),.Clk(CLK18_432M),.Reset(~ZRESET));
//dac dac(.DACout(dac_out),.DACin(wave_out),.Clk(CLK36_384M),.Reset(1'b0));

// DAC output PIN assign
OBUF_F_24 DACL (.I(dac_out), .O(DAC_L));
OBUF_F_24 DACR (.I(dac_out), .O(DAC_R));

// check
//output Z80_RD  = ZRD;
//output Z80_CLK = Z80CLK;

endmodule

